/*
================================================================================
 Plik:          buffers.c
--------------------------------------------------------------------------------
 Autor:         ukasz Krzak
 Projekt:       modu uniwersalny
 Kompilator:    AVR GCC 3.4.1
 Hardware:      AVR, ARM
 Dokumentacja:  doxygen 1.4.3
--------------------------------------------------------------------------------
 Zawiera:       procedury implementujace bufory kolowe
--------------------------------------------------------------------------------
 Komentarze:    maksymalny rozmiar buforow jest ustalany poprzez definicje typu
                tbuf_size, ktora znajduje sie w pliku buffers.h
              
                tekst kodu sformatowany
================================================================================
*/


#include "config.h"
//#include "includes.h"

#include <stdlib.h>
#include "buffers.h"

#if (BUF_USE_ERROR_CHECK == 1)
// Wskanik do funkcji wywoywanej w przypadku wystpienia bdu w obsudze buforw
static void (*ErrorFunc_Callback) (TBuffer*, uint8_t);
#endif

extern void * BUF_MEMORY_MALLOC_FUNC(size_t);
extern void BUF_MEMORY_FREE_FUNC(void*);

// -----------------------------------------------------------------------------
//  BUF_Create
//
/// Funkcja tworzy bufor kolowy o zadanym rozmiarze. Zwraca wskaznik do opisu
/// struktury bufora (ktory jest od teraz uchwytem bufora) lub NULL jesli tworzenie
/// sie nie powiodlo.
// -----------------------------------------------------------------------------
/*TBuffer *BUF_Create(tbuf_size BufferSize)
{
    TBuffer *NewBuffer;

    // alokacja pamieci na deskryptor bufora
    NewBuffer = BUF_MEMORY_MALLOC_FUNC(sizeof(TBuffer));
    // sprawdzenie alokacji
    if (NewBuffer == NULL) {
        return NULL;
    }
    // alokacja pamieci na bufor
    NewBuffer->BufferPtr = BUF_MEMORY_MALLOC_FUNC(BufferSize);
    // sprawdzenie alokacji
    if (NewBuffer->BufferPtr == NULL) {
        // Jesli sie nie powiodla to zwalniamy pamiec na opis bufora
        BUF_MEMORY_FREE_FUNC(NewBuffer);
        return NULL;
    }
    // ustawiamy wskazniki i domyslne ustawienia
    NewBuffer->Size = BufferSize;
    NewBuffer->WritePtr = 0;
    NewBuffer->ReadPtr = 0;
    NewBuffer->Count = 0;

#if (BUF_USE_TRAPS == 1)
    NewBuffer->ReadTrapFunc = NULL;
    NewBuffer->WriteTrapFunc = NULL;
    NewBuffer->ReadTrapNo = (tbuf_size)0xffffffff;
    NewBuffer->WriteTrapNo = (tbuf_size)0xffffffff;
#endif

    // Jesli wszystko sie udalo zwracamy wskaznik do opisu bufora
    return NewBuffer;

} //*/                              /* BUF_Create */


// -----------------------------------------------------------------------------
//  BUF_CreateOverExistingArea
//
/// Funkcja tworzy strukture bufora kolowego na juz istniejacym obszarze pamieci
/// Zwraca wskaznik do opisu struktury bufora (ktory jest od teraz uchwytem bufora)
/// lub NULL jesli tworzenie sie nie powiodlo.
// -----------------------------------------------------------------------------
void BUF_CreateOverExistingArea(TBuffer *NewBuffer, uint8_t * BufPtr, tbuf_size BufferSize)
{
    // sprawdzenie alokacji
    if ((BufPtr == NULL) || (NewBuffer == NULL)) {
        return;
    }
    // przepisanie adresu obszaru pamieci jako wskaznik do bufora
    NewBuffer->BufferPtr = BufPtr;
    // ustawiamy wskazniki i domyslne ustawienia
    NewBuffer->Size = BufferSize;
    NewBuffer->WritePtr = 0;
    NewBuffer->ReadPtr = 0;
    NewBuffer->Count = 0;

#if (BUF_USE_TRAPS == 1)
    NewBuffer->ReadTrapFunc = NULL;
    NewBuffer->WriteTrapFunc = NULL;
    NewBuffer->ReadTrapNo = (tbuf_size)0xffffffff;
    NewBuffer->WriteTrapNo = (tbuf_size)0xffffffff;
#endif

}                               /* BUF_CreateOverExistingArea */


// -----------------------------------------------------------------------------
//  BUF_Write
//
/// Funkcja zapisuje dane o zadanej dugoci do wybranego bufora.
/// Zwraca:                                                                   \n
///    BUFFER_OK               ,jesli wszystko sie udalo                      \n
///    BUFFER_NOT_INITIALIZED  ,jesli bufor nie zainicjalizowany              \n
///    BUFFER_OVERRUN          ,jesli wystapilo przepelnienie bufora          \n
// -----------------------------------------------------------------------------
uint8_t BUF_Write(TBuffer * Buffer, uint8_t * Source, tbuf_size Count)
{
    tbuf_size i;
    int tempSREG;

#if (BUF_FAST_BUFFERS == 0)
    // sprawdzenie czy deskrytpor poprawny
    if (Buffer == NULL) {
        if (NULL != ErrorFunc_Callback) {
            // wywoanie funkcji obsugi bdu
            ErrorFunc_Callback(Buffer, BUFFER_NOT_INITIALIZED);
        }
        return BUFFER_NOT_INITIALIZED;
    }
    // sprawdzenie czy bufor zainicjalizowany
    if (Buffer->BufferPtr == NULL) {
        if (NULL != ErrorFunc_Callback) {
            // wywoanie funkcji obsugi bdu
            ErrorFunc_Callback(Buffer, BUFFER_NOT_INITIALIZED);
        }
        return BUFFER_NOT_INITIALIZED;
    }
#endif

    for (i = 0; i < Count; i++) {

#if (BUF_USE_ERROR_CHECK == 1)
        // Sprawdzenie czy nie wystapilo przepelnienie
        if (Buffer->Count >= Buffer->Size) {

            if (NULL != ErrorFunc_Callback) {
                // wywoanie funkcji obsugi bdu
                ErrorFunc_Callback(Buffer, BUFFER_OVERRUN);
            }
            return BUFFER_OVERRUN;
        }
#endif
        // sekcja krytyczna - musi byc zdefiniowana, aby podczas zmiany opisu bufora nie
        // nastapila jego modyfikacja
        ENTER_CRITICAL(tempSREG);

        // Zapis bajtu
        Buffer->BufferPtr[Buffer->WritePtr] = Source[i];
        // Przesuniecie wskaznika zapisu
        Buffer->WritePtr++;
        if (Buffer->WritePtr == Buffer->Size) {
            Buffer->WritePtr = 0;
        }
        // Zwiekszenie licznika bajtow w buforze
        Buffer->Count++;
        // Koniec sekcji krytycznej
        EXIT_CRITICAL(tempSREG);
    }


    return BUFFER_OK;
}                               /* BUF_Write */


// -----------------------------------------------------------------------------
//  BUF_WriteByte
//
/// Funkcja zapisuje jeden bajt do wybranego bufora.
/// Zwraca:                                                                   \n
///    BUFFER_OK               ,jesli wszystko sie udalo                      \n
///    BUFFER_NOT_INITIALIZED  ,jesli bufor nie zainicjalizowany              \n
///    BUFFER_OVERRUN          ,jesli wystapilo przepelnienie bufora          \n
// -----------------------------------------------------------------------------
uint8_t BUF_WriteByte(TBuffer * Buffer, uint8_t Byte)
{
    int tempSREG;

#if (BUF_FAST_BUFFERS == 0)
    // sprawdzenie czy deskrytpor poprawny
    if (Buffer == NULL) {
        if (NULL != ErrorFunc_Callback) {
            // wywoanie funkcji obsugi bdu
            ErrorFunc_Callback(Buffer, BUFFER_NOT_INITIALIZED);
        }
        return BUFFER_NOT_INITIALIZED;
    }
    // sprawdzenie czy bufor zainicjalizowany
    if (Buffer->BufferPtr == NULL) {
        if (NULL != ErrorFunc_Callback) {
            // wywoanie funkcji obsugi bdu
            ErrorFunc_Callback(Buffer, BUFFER_NOT_INITIALIZED);
        }
        return BUFFER_NOT_INITIALIZED;
    }
#endif

#if (BUF_USE_ERROR_CHECK == 1)
    // Sprawdzenie czy nie wystapilo przepelnienie
    if (Buffer->Count >= Buffer->Size) {
        if (NULL != ErrorFunc_Callback) {
            // wywoanie funkcji obsugi bdu
            ErrorFunc_Callback(Buffer, BUFFER_OVERRUN);
        }
        // zerujemy bufor
        Buffer->ReadPtr = Buffer->WritePtr;
        Buffer->Count = 0;
        return BUFFER_OVERRUN;
    }
#endif

    // sekcja krytyczna - musi byc zdefiniowana, aby podczas zmiany opisu bufora nie
    // nastapila jego modyfikacja
    ENTER_CRITICAL(tempSREG);


    // Zapis bajtu
    Buffer->BufferPtr[Buffer->WritePtr] = Byte;
    // Przesuniecie wskaznika zapisu
    Buffer->WritePtr++;
    if (Buffer->WritePtr >= Buffer->Size) {
        Buffer->WritePtr = 0;
    }
    // Zwiekszenie licznika bajtow w buforze
    Buffer->Count++;

    // koniec sekcji krytycznej
    EXIT_CRITICAL(tempSREG);

#if (BUF_USE_TRAPS == 1)
    // Sprawdzenie czy nie wystpia puapka zapisu
    if (NULL != Buffer->WriteTrapFunc) {
        if (Buffer->WriteTrapNo) {
            Buffer->WriteTrapNo--;
        } else {
            Buffer->WriteTrapFunc();
            BUF_ClrWriteTrap(Buffer);
        }
    }
    
#endif


    return BUFFER_OK;
}                               /* BUF_WriteByte */


// -----------------------------------------------------------------------------
//  BUF_Read
//
/// Funkcja odczytuje dane o zadanej dlugosci z wybranego bufora.
/// Zwraca:                                                                   \n
///    BUFFER_OK               ,jesli wszystko sie udalo.                     \n
///    BUFFER_NOT_INITIALIZED  ,jesli bufor niezainicjalizowany.              \n
///    BUFFER_EMPTY            ,jesli bufor zostal oprozniony w trakcie odczytu.
// -----------------------------------------------------------------------------
uint8_t BUF_Read(TBuffer * Buffer, uint8_t * Destination, tbuf_size Count)
{
    int tempSREG;

#if (BUF_FAST_BUFFERS == 0)
    // sprawdzenie czy deskrytpor poprawny
    if (Buffer == NULL) {
        if (NULL != ErrorFunc_Callback) {
            // wywoanie funkcji obsugi bdu
            ErrorFunc_Callback(Buffer, BUFFER_NOT_INITIALIZED);
        }
        return BUFFER_NOT_INITIALIZED;
    }
    // sprawdzenie czy bufor zainicjalizowany
    if (Buffer->BufferPtr == NULL) {
#if (BUF_USE_ERROR_CHECK == 1)
        if (NULL != ErrorFunc_Callback) {
            // wywoanie funkcji obsugi bdu
            ErrorFunc_Callback(Buffer, BUFFER_NOT_INITIALIZED);
        }
#endif
        return BUFFER_NOT_INITIALIZED;
    }
#endif
    // sekcja krytyczna - musi byc zdefiniowana, aby podczas odczytu bufora nie
    // nastapila jego modyfikacja
    ENTER_CRITICAL(tempSREG);

    while (Count && Buffer->Count) {
        // kopiujemy bajt
        *Destination = Buffer->BufferPtr[Buffer->ReadPtr];
        Destination++;
        Count--;

        // zmniejszamy liczbe bajtow w buforze
        Buffer->Count--;

        // przesuwamy wskaznik odczytu
        Buffer->ReadPtr++;
        if (Buffer->ReadPtr == Buffer->Size) {
            Buffer->ReadPtr = 0;
        }
    }                           // for

    // Koniec sekcji krytycznej
    EXIT_CRITICAL(tempSREG);

    // Jesli bufor zostal oprozniony, zwracamy odpowiedni kod
    if (Buffer->Count == 0) {
        return BUFFER_EMPTY;
    }
    // Jesli wszystko sie udalo zwracamy kod sukcesu
    return BUFFER_OK;

}                               /* ReadBuffer */


// -----------------------------------------------------------------------------
//   BUF_ReadByte
//
/// Szybka funkcja do pobierania jednego bajtu z wybranego bufora.
///
// -----------------------------------------------------------------------------
uint8_t BUF_ReadByte(TBuffer * Buffer)
{
    uint8_t Byte;
    int tempSREG;
    
    if (!Buffer->Count) {
        // TODO
        return BUF_DEFAULT_CHARACTER;
    }
    // sekcja krytyczna - musi byc zdefiniowana, aby podczas odczytu bufora nie
    // nastapila jego modyfikacja
    ENTER_CRITICAL(tempSREG);


    // Odczyt bajtu
    Byte = Buffer->BufferPtr[Buffer->ReadPtr];
    // Zmniejszenie licznika bajtow
    Buffer->Count--;
    // Uaktualnienie wskaznika odczytu
    Buffer->ReadPtr++;
    if (Buffer->ReadPtr == Buffer->Size) {
        Buffer->ReadPtr = 0;
    }

    // Koniec sekcji krytycznej
    EXIT_CRITICAL(tempSREG);

#if (BUF_USE_TRAPS == 1)
    // Sprawdzenie czy nie wystpia puapka odczytu
    if (NULL != Buffer->ReadTrapFunc) {
        Buffer->ReadTrapNo--;
        if (!Buffer->ReadTrapNo) {
            Buffer->ReadTrapFunc();
        }
    }
  
#endif

    return Byte;

}                               /* BUF_ReadByte */


// -----------------------------------------------------------------------------
//   BUF_GetCount
//
/// Funkcja zwraca aktualna liczbe bajtow w buforze
///
// -----------------------------------------------------------------------------
tbuf_size BUF_GetCount(TBuffer * Buffer)
{
    return Buffer->Count;

}                               /* BUF_GetCount */


// -----------------------------------------------------------------------------
//   BUF_GetFreeSpace
//
///  Funkcja zwraca ilosc wolnego miejsca w buforze.
///
// -----------------------------------------------------------------------------
tbuf_size BUF_GetFreeSpace(TBuffer * Buffer)
{
    return Buffer->Size - Buffer->Count;

}                               /* BUF_GetFreeSpace */


// -----------------------------------------------------------------------------
//  BUF_Clear
//
/// Funkcja czysci zadany bufor.
///
// -----------------------------------------------------------------------------
void BUF_Clear(TBuffer * Buffer)
{
    int tempSREG;

    // sekcja krytyczna - musi byc zdefiniowana, aby podczas zmiany opisu bufora nie
    // nastapila jego modyfikacja
    ENTER_CRITICAL(tempSREG);

    Buffer->WritePtr = 0;
    Buffer->ReadPtr = Buffer->WritePtr;
    Buffer->Count = 0;

    // Koniec sekcji krytycznej
    EXIT_CRITICAL(tempSREG);

}                               /* BUF_Clear */


#if (BUF_USE_ERROR_CHECK == 1)
// -----------------------------------------------------------------------------
//  BUF_SetErrorFuncCallback
//
/// Funkcja zapisuje wskanik do funkcji ktra bdzie wywoywana w przypadku 
/// wystpienia bdu w obsudze buforw
///
// -----------------------------------------------------------------------------
void BUF_SetErrorFuncCallback(void (*ErrorFunc_callback) (TBuffer*, uint8_t))
{
    ErrorFunc_Callback = ErrorFunc_callback;
} /* BUF_SetErrorFuncCallback */

#endif //BUF_USE_ERROR_CHECK


#if (BUF_USE_TRAPS == 1)
// -----------------------------------------------------------------------------
//  BUF_SetReadTrap
//
/// Funkcja zapisuje wskanik do funkcji ktra bdzie wywoywana w przypadku 
/// wystpienia puapki odczytu.
///
// -----------------------------------------------------------------------------
void BUF_SetReadTrap(TBuffer *Buffer, tbuf_size ByteNo, void (*ReadTrapFunc_callback) (void))
{
    Buffer->ReadTrapFunc = ReadTrapFunc_callback;
    Buffer->ReadTrapNo = ByteNo;
} /* BUF_SetReadTrap */


// -----------------------------------------------------------------------------
//  BUF_ClrReadTrap
//
/// Funkcja kasuje wskanik do funkcji ktra bdzie wywoywana w przypadku 
/// wystpienia puapki zapisu, wyczajc przez to puapk.
///
// -----------------------------------------------------------------------------
void BUF_ClrReadTrap(TBuffer *Buffer)
{
    Buffer->ReadTrapFunc = NULL;
    Buffer->ReadTrapNo = (tbuf_size)0xffffffff;
} /* BUF_ClrReadTrap */


// -----------------------------------------------------------------------------
//  BUF_SetWriteTrap
//
/// Funkcja zapisuje wskanik do funkcji ktra bdzie wywoywana w przypadku 
/// wystpienia puapki zapisu.
///
// -----------------------------------------------------------------------------
void BUF_SetWriteTrap(TBuffer *Buffer, tbuf_size ByteNo, void (*WriteTrapFunc_callback) (void))
{
    Buffer->WriteTrapFunc = WriteTrapFunc_callback;
    Buffer->WriteTrapNo = ByteNo;
} /* BUF_SetWriteTrap */


// -----------------------------------------------------------------------------
//  BUF_ClrWriteTrap
//
/// Funkcja kasuje wskanik do funkcji ktra bdzie wywoywana w przypadku 
/// wystpienia puapki zapisu, wyczajc przez to puapk.
///
// -----------------------------------------------------------------------------
void BUF_ClrWriteTrap(TBuffer *Buffer)
{
    Buffer->WriteTrapFunc = NULL;
    Buffer->WriteTrapNo = (tbuf_size)0xffffffff;
} /* BUF_ClrWriteTrap */


#endif //BUF_USE_TRAPS



/*! \file buffers.c
    \brief Biblioteka funkcji implementujacych bufory koowe.

    Biblioteka buffers zawiera funkcje i definicje typow implementujace bufory
    kolowe, operujace na obszarze pamieci przydzielanym dynamicznie lub statycznie.
*/
