/*
================================================================================
 Plik:          buffers.h
--------------------------------------------------------------------------------
 Autor:         ukasz Krzak
 Projekt:       modu uniwersalny
 Kompilator:    AVR GCC 3.4.1
 Hardware:      AVR, ARM
 Dokumentacja:  doxygen 1.4.3
--------------------------------------------------------------------------------
 Zawiera:       procedury implementujace bufory kolowe
--------------------------------------------------------------------------------
 Komentarze:
                tekst kodu sformatowany
================================================================================
*/


#ifndef BUFFERS_H
/// @cond DOXYGEN_IGNORE
#define BUFFERS_H
#define BUFFERS_VER_HI    1
#define BUFFERS_VER_LO    2
/// @endcond DOXYGEN_IGNORE
//#include "config.h"
#include <stdint.h>


/** \defgroup buffers Bufory koowe oglnego zastosowania
    \code
    #include "buffers.h"
    \endcode
   
    Biblioteka zawiera funkcje i definicje typw implementujce bufory koowe, operujce na obszarze 
    pamici przydzielanym dynamicznie lub statycznie.

    Pojedynczy bufor koowy skada si ze struktury opisujcej ten bufor, oraz obszaru pamici 
    przydzielonego na uytek tego bufora. Kady bufor jest identyfikowany przez swj uchwyt, 
    przy czym przez uchwyt bufora koowego rozumie si wskanik do struktury opisujcej ten bufor (\ref TBuffer). 

    Biblioteka umoliwia zmian typu zmiennych opisujcych wielko bufora, poprzez zmian definicji 
    typu tbuf_size (domylnie jest to uint8_t) 
    \code
    #define tbuf_size   uint8_t
    \endcode
    Biblioteka umoliwia wykluczenie z kompilacji procedur sprawdzajcych poprawno uchwytw buforw, 
    co przyspiesza dziaanie funkcji bibliotecznych. Aby wprowadzi to pomijanie naley zdefiniowa makro 
    FAST_BUFFERS (domylnie zdefiniowane).
    \code    
    #define FAST_BUFFERS
    \endcode

    Biblioteka buffers posiada 2 funkcje umoliwiajce tworzenie buforw koowych. Jedna operuje na buforach 
    tworzonych dynamicznie, druga na statycznie zdefiniowanych obszarach pamici. Biblioteka posiada jedn 
    funkcj umoliwiajc zapis bajtu do bufora, oraz dwie wersje funkcje odczytujcych dane z bufora (jedna 
    zoptymalizowana pod ktem odczytu tylko jednego bajtu, druga umoliwia jednorazowy odczyt dowolnej iloci 
    bajtw). Dostpne s take: funkcje zwracajce aktualn ilo bajtw w buforze, ilo wolnego miejsca w 
    buforze oraz funkcja czyszczca bufor.

*/

/*@{*/

/** \name Definicje konfiguracji bilioteki
@{
*/

#ifndef BUF_FAST_BUFFERS
/**
	Jeli makro to jest zdefiniowane na 1 to przy kompilacji zostanie pominita cz
 	kodu odpowiedzialna za sprawdzenie poprawnoci niektrych
	argumentw funkcji zapisujcych i odczytujcych.
*/
#define BUF_FAST_BUFFERS                    1
#endif 

#ifndef BUF_USE_ERROR_CHECK
/**
	Jeli makro to jest zdefiniowane na 1 to doczona zostanie moliwo wywoania funkcji uytkownika w przypadku wystpienia
    bedu bufora
*/
#define BUF_USE_ERROR_CHECK                 0
#endif

#ifndef BUF_USE_TRAPS
/**
	Jeli makro to jest zdefiniowane to doczona zostanie moliwo ustawiania puapek na konkretny numer bajtu w buforze.
    Puapka taka umoliwia wywoanie funkcji uytkownika w momencie zapisu lub odczytu bajtu o konkretnym numerze.
    Przykadowo, uytkownik moe wywoa funckcj zewntrzn po otrzymaniu 10bajtw, lub np. wstrzyma wysyanie po 5 bajcie.
*/
#define BUF_USE_TRAPS                       0
#endif

/**
	Definiuje domylny znak zwracany przez funkcj \ref BUF_ReadByte jeli bufor jest pusty.
*/
#define BUF_DEFAULT_CHARACTER               0x55


/*@}*/

/** \name Definicje rezultatow funkcji
@{*/
#define BUFFER_OK                           0
#define BUFFER_OUT_OF_MEMORY                10
#define BUFFER_NOT_INITIALIZED              11
#define BUFFER_OVERRUN                      12
#define BUFFER_EMPTY                        13
/*@}*/

/**
	Typ okrelajcy rozmiary buforw.\n
	Pozwala zdefiniowa typ zmiennych opisujcych wielko bufora. Domylnym typem jest uint8_t.
*/
typedef uint16_t tbuf_size;

/**
    \brief Strutura opisujca bufor koowy.

	Struktura opisujca bufor koowy. Wskanik do tej struktury traktuje si
	jako uchwyt bufora. Jest on parametrem identyfikujcym bufor w wikszosci
	wywoa funkcji z tego moduu.
*/
typedef struct {
    uint8_t *BufferPtr;                     ///< wskaznik do fizycznego obszaru pamieci
    volatile tbuf_size Size;                ///< rozmiar bufora
    volatile tbuf_size WritePtr;            ///< indeks zapisu (zawsze wskazuje aktualna pozycje zapisu)
    volatile tbuf_size ReadPtr;             ///< indeks odczytu (zawsze wskazuje aktualna pozycje odczytu)
    volatile tbuf_size Count;               ///< aktualna liczba bajtow w buforze
#ifdef BUF_USE_TRAPS
    volatile tbuf_size ReadTrapNo;          ///< numer bajtu, ktrego odczyt spowoduje wystpienie puapki odczytu
    volatile tbuf_size WriteTrapNo;         ///< numer bajtu, ktrego odczyt spowoduje wystpienie puapki zapisu
    void (*ReadTrapFunc)(void);    ///< wskanik do funkcji wywoywanej w przypadku wystpienia puapki odczytu
    void (*WriteTrapFunc)(void);   ///< wskanik do funkcji wywoywanej w przypadku wystpienia puapki odczytu
#endif
} TBuffer;

/**
	Typ wskanika do bufora.
*/
typedef TBuffer* PBuffer;

/**
        Tworzy bufor koowy wykorzystujc makro BUF_MEMORY_MALLOC_FUNC do dynamicznej alokacji obszaru pamici 
        o rozmiarze BufferSize. Funkcja alokuje rwnie pami na struktur opisu bufora i zwraca wskanik do 
        tej struktury, jako uchwyt bufora, lub \c NULL, jeli bufor nie moe zosta utworzony (np. z powodu braku 
        pamici).
        \param BufferSize       dany rozmiar bufora
	    \return Wskanik do opisu struktury bufora (ktry jest od teraz uchwytem bufora)
		lub \c NULL jeli tworzenie si nie powiodo.
*/
TBuffer* BUF_Create(tbuf_size BufferSize);

/**
	    Tworzy bufor koowy wykorzystujc istniejcy obszar (lub fragment obszaru) pamici wskazywany przez 
        BufPtr o rozmiarze BufferSize. Dynamicznie alokowana jest tylko pami na struktur opisu bufora. 
        Funkcja zwraca wskanik do tej struktury, jako uchwyt bufora, lub \c NULL, jeli bufor nie moe zosta 
        utworzony (np. z powodu braku pamici).
        \param BufPtr           wskanik do istniejcego obszaru pamici na ktrym utworzony zostanie bufor koowy
        \param BufferSize       dany rozmiar bufora
        \return Wskanik do opisu struktury bufora (ktry jest od teraz uchwytem bufora)
		lub \c NULL jesli tworzenie si nie powiodo.
*/
void BUF_CreateOverExistingArea(TBuffer *NewBuffer, uint8_t *BufPtr, tbuf_size BufferSize);

/**
        Zapisuje zadan przez parametr Count liczb bajtw z obszaru wskazywanego przez parametr Source
        do bufora identyfikowanego przez uchwyt Buffer. Uaktualniana jest liczba bajtw w buforze oraz 
        przesuwany jest indeks zapisu bufora. 
        \param Buffer           uchwyt bufora
        \param Source           wskanik do obszaru skd skopiowane zostan dane
        \param Count            liczba bajtw do zapisu
        \return
        \c BUFFER_OK                - jeli wszystko si udao\n
        \c BUFFER_NOT_INITIALIZED   - jeli bufor nie zainicjalizowany\n
        \c BUFFER_OVERRUN           - jeli wystapio przepenienie bufora
*/
uint8_t BUF_Write(TBuffer * Buffer, uint8_t * Source, tbuf_size Count);


/**
        Zapisuje pojedynczy bajt zadany przez parametr Byte do bufora identyfikowanego przez uchwyt Buffer. 
        Uaktualniana jest liczba bajtw w buforze oraz przesuwany jest indeks zapisu bufora. 
        \param Buffer           uchwyt bufora
        \param Byte             bajt do zapisu
        \return
        \c BUFFER_OK                - jeli wszystko si udao\n
        \c BUFFER_NOT_INITIALIZED   - jeli bufor nie zainicjalizowany\n
        \c BUFFER_OVERRUN           - jeli wystapio przepenienie bufora
*/
uint8_t BUF_WriteByte(TBuffer *Buffer, uint8_t Byte);


/**
	    Odczytuje zadan przez parametr Count liczb bajtw z bufora identyfikowanego uchwytem Buffer kopiujc 
        je do obszaru zadanego przez wskanik Destination. Przesuwany jest indeks zapisu oraz uaktualniana 
        jest liczba bajtw w buforze  oznacza to e bajty odczytane przestaj istnie w buforze! 
        \param Buffer           uchwyt bufora
        \param Destination      wskanik do obszaru gdzie skopiowane zostan dane z bufora
        \param Count            liczba bajtw do odczytu
	    \return
    	\c BUFFER_OK               - jeli wszystko sie udao\n
	    \c BUFFER_NOT_INITIALIZED  - jeli bufor niezainicjalizowany\n
    	\c BUFFER_EMPTY            - jeli bufor zosta oprniony w trakcie odczytu
*/
uint8_t BUF_Read(TBuffer *Buffer, uint8_t *Destination, tbuf_size Count);

/**
	    Pobiera jeden bajt z bufora identyfikowanego uchwytem Buffer. Przesuwany jest indeks zapisu oraz 
        uaktualniana jest liczba bajtw w buforze  oznacza to e bajty odczytane przestaj istnie w buforze! 
        Funkcja zwraca odczytany bajt. Jeli bufor jest pusty - zawsze zwraca 0.
        \param Buffer           uchwyt bufora
        \return odczytany bajt, lub 0, jeli bufor pusty
*/
uint8_t BUF_ReadByte(TBuffer *Buffer);

/**
	    Zwraca aktualn liczb bajtw zapisanych w buforze.
	    \return Liczba bajtw wpisanych do bufora
*/
tbuf_size BUF_GetCount(TBuffer *Buffer);

/**
	    Zwraca ilo wolnego miejsca w buforze.
	    \return Liczba bajtw moliwych do wpisania do bufora
*/
tbuf_size BUF_GetFreeSpace(TBuffer *Buffer);

/**
        Czyci zadany bufor. Jej dziaanie polega na zrwnaniu indeksw zapisu i odczytu, 
        oraz ustawieniu licznika bajtw w buforze na 0.
*/
void BUF_Clear(TBuffer *Buffer);


#if (BUF_USE_ERROR_CHECK == 1)

/**
        Ustawia wskanik do funkcji wywoywanej w przypadku wystpienia bdu obsugi bufora
        \param ErrorFunc_callback      wskanik do funkcji (lub NULL)
*/
void BUF_SetErrorFuncCallback(void (*ErrorFunc_callback) (TBuffer*, uint8_t));

#endif // BUF_USE_ERROR_CHECK


#if (BUF_USE_TRAPS == 1)

/**
        Ustawia wskanik do funkcji wywoywanej w przypadku wystpienia puapki odczytu.
        \param Buffer                  wskanik do bufora dla ktrego ma zosta ustawiona puapka
        \param ByteNo                  numer bajtu ktry ma wywoa puapk
        \param ReadTrapFunc_callback   wskanik do funkcji (lub NULL)
*/
void BUF_SetReadTrap(TBuffer *Buffer, tbuf_size ByteNo, void (*ReadTrapFunc_callback) (void));

/**
        Kasuje puapk odczytu dla zadanego bufora
        \param Buffer                  wskanik do bufora dla ktrego ma zosta skasowana puapka
*/
void BUF_ClrReadTrap(TBuffer *Buffer);

/**
        Ustawia wskanik do funkcji wywoywanej w przypadku wystpienia puapki zapisu.
        \param Buffer                  wskanik do bufora dla ktrego ma zosta ustawiona puapka
        \param ByteNo                  numer bajtu ktry ma wywoa puapk
        \param WriteTrapFunc_callback   wskanik do funkcji (lub NULL)
*/
void BUF_SetWriteTrap(TBuffer *Buffer, tbuf_size ByteNo, void (*WriteTrapFunc_callback) (void));

/**
        Kasuje puapk zapisu dla zadanego bufora
        \param Buffer                  wskanik do bufora dla ktrego ma zosta skasowana puapka
*/
void BUF_ClrWriteTrap(TBuffer *Buffer);

#endif // BUF_USE_TRAPS

/*@}*/

/*! \file buffers.h
    \brief Biblioteka funkcji implementujacych bufory koowe.

    Biblioteka buffers zawiera funkcje i definicje typow implementujace bufory
    kolowe, operujace na obszarze pamieci przydzielanym dynamicznie lub statycznie.
*/

#endif
